//////////////////////////////////////////////////////////////////////NVMH3////
// File:  $Id: //devrel/Playpen/kbjorke/fx_scripts/scatter_scene.cs#2 $
// 
// Copyright NVIDIA Corporation 2004
// TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
// *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
// OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
// AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
// BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
// WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
// BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
// ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
// BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
//
// Comments:
// Builds a spiral of materials.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Xml;
using System.Collections;
using System.IO;
using System.Windows.Forms;
using interop.nv_sys;
using System.Runtime.InteropServices;


namespace PluginList
{
	public class Simple : INVUtility
	{
		// Report an error from the script
		private void ReportError(INVSystem Sys, string Text)
		{
			NVReport Rep = new NVReportClass();
			Rep.SetReport(eNVREPORT.NVREPORT_ERROR, Text, "", -1);
			Sys.Report(Rep);
		}

		// Report a log from the script
		private void Report(INVSystem Sys, string Text)
		{
			NVReport Rep = new NVReportClass();
			Rep.SetReport(eNVREPORT.NVREPORT_LOG, Text, "", -1);
			Sys.Report(Rep);
		}
		
		public string GetText(INVString str)
		{
			string strOut;
			str.GetString(out strOut);
			return strOut;
		}
			
		// make an NVstring from a string
		NVString CreateNVString(string In)
		{
			NVString str = new NVString();
			str.SetString(In);
			return str;
		}
		
		// random 3d location within +/-(span/2)
		public void NVScatter3(float span,Random r,out NVFloat3 result)
		{
			result.x = span * ((float)r.NextDouble() - 0.5f);
			result.y = span * ((float)r.NextDouble() - 0.5f);
			result.z = span * ((float)r.NextDouble() - 0.5f);
		}
		
		// Multiply quaternions
		public void NVQuatMult(NVQuat a, NVQuat b, out NVQuat result)
		{

			result.w = a.w*b.w - a.x*b.x - a.y*b.y - a.z*b.z;
			result.x = a.w*b.x + a.x*b.w + a.y*b.z - a.z*b.y;
			result.y = a.w*b.y + a.y*b.w + a.z*b.x - a.x*b.z;
			result.z = a.w*b.z + a.z*b.w + a.x*b.y - a.y*b.x;
		}

		public NVQuat XrotQuat(float a) 
		{
			NVQuat q;
			float a2 = a/2;
			q.x = (float)Math.Cos(a2);
			q.y = (float)Math.Sin(a2);
			q.z = 0;
			q.w = 0;
			return q;
		}
		public NVQuat YrotQuat(float a) 
		{
			NVQuat q;
			float a2 = a/2;
			q.x = (float)Math.Cos(a2);
			q.z = (float)Math.Sin(a2);
			q.y = 0;
			q.w = 0;
			return q;
		}
		public NVQuat ZrotQuat(float a) 
		{
			NVQuat q;
			float a2 = a/2;
			q.x = (float)Math.Cos(a2);
			q.w = (float)Math.Sin(a2);
			q.z = 0;
			q.y = 0;
			return q;
		}

		// randowm quaternion
		public void NVRandomQuat(Random r,out NVQuat result)
		{
			float ax = 3.14159265358979f * (float)r.NextDouble();
			float ay = 3.14159265358979f * (float)r.NextDouble();
			float az = 3.14159265358979f * (float)r.NextDouble();
			NVQuat qx = XrotQuat(ax);
			NVQuat qy = YrotQuat(ay);
			NVQuat qz = ZrotQuat(az);
			NVQuat qa;
			NVQuatMult(qx,qy,out qa);
			NVQuatMult(qa,qz,out result);
		}
		
		// Bezier function on NVFloat3's
		public void Bezier3(NVFloat3 a,NVFloat3 b,NVFloat3 c,NVFloat3 d,
					float t,out NVFloat3 result)
		{
			float nt = 1.0f - t;
			float b0 = nt * nt * nt;
			float b1 = 3.0f * t * nt * nt;
			float b2 = 3.0f * t * t * nt;
			float b3 = t * t * t;
			result.x = a.x*b0 + b.x*b1 + c.x*b2 + d.x*b3;
			result.y = a.y*b0 + b.y*b1 + c.y*b2 + d.y*b3;
			result.z = a.z*b0 + b.z*b1 + c.z*b2 + d.z*b3;
		}

		// get a list of FX files
		public string[] GetFiles()
		{
    		OpenFileDialog openFileDialog1 = new OpenFileDialog();
		    openFileDialog1.InitialDirectory = "" ;
		    openFileDialog1.FileName="";
    		openFileDialog1.Filter = ".fx files for surfaces(*.fx)|*.fx|All files (*.*)|*.*" ;
    		openFileDialog1.FilterIndex = 1 ;
    		openFileDialog1.RestoreDirectory = false ;
    		openFileDialog1.Multiselect = true;
    		openFileDialog1.Title = "Select a list of .fx files to preview";
    		if(openFileDialog1.ShowDialog() == DialogResult.OK)
    		{
    			return openFileDialog1.FileNames;
    		}
    		return null;
    	}


		// Script entry point.
		public int Run(interop.nv_sys.NVSystem Sys)
		{
			// Get the scene
			NVScene Scene;
			Sys.GetScene(out Scene);
			Scene.Reset();
			Scene.RemoveAllMaterials();
			NVMaterial Mat;
			// Load all object script based materials
			string[] effects = GetFiles();
			if (effects == null)
				return 0;
			foreach(string effect in effects)
			{
				NVString MatName;
				Sys.GenerateUniqueName(CreateNVString("material"),
										CreateNVString(effect),
										out MatName);
				Scene.CreateMaterial(MatName, CreateNVString(effect), out Mat);
			}

			ArrayList Array = new ArrayList();
			// Cull all that aren't appropriate
			for (uint Current = 0; Current < Scene.GetNumMaterials(); Current++)
			{
				Scene.GetMaterial(Current, out Mat);
				INVMedia Media = (INVMedia)Mat;
				// Get the script info
				NVScriptExecute scriptExecute;
				Mat.GetScriptExecute(out scriptExecute);
				if (scriptExecute == null)
				{
					Array.Add(Mat);
				}
				Report(Sys, "checking2");
				NVScriptData scriptData = null;
				scriptExecute.GetEntryPoint(out scriptData);
				if (scriptData == null)
				{
					Array.Add(Mat);
				}
				NVSCRIPT_CLASS scriptClass = 0;
				scriptData.GetClass(out scriptClass);
				Report(Sys, scriptClass.ToString());
				// Apply the override material
				// Only care about object script materials.
				if (scriptClass != NVSCRIPT_CLASS.NVSCRIPTCLASS_OBJECT)
				{
					NVString Name;
					Media.GetMediaName(out Name);
					string strName;
					Name.GetString(out strName);
					string Text = "Can't use ";
					Text += strName;
					Text += ", it isn't an object scripted material";
					ReportError(Sys, Text);
					Array.Add(Mat);
					continue;
				}
			}
			// We remove materials seperately, because we don't want to 
			// corrupt the scene material count.
			for (int MatNum = 0; MatNum < Array.Count; MatNum++)
			{
				Scene.RemoveMaterial((NVMaterial)Array[MatNum]);
			}

			Random rnd = new Random();
			for (uint Current = 0; Current < Scene.GetNumMaterials(); Current++)
			{
				Scene.GetMaterial(Current, out Mat);
				INVMedia Media = (INVMedia)Mat;
				// New Geometry pipe.
				INVNode PipeNode;
				INVGeoPipe GeoPipe;
				Guid IID_GeoPipe = Marshal.GenerateGuidForType(typeof(NVGeoPipeClass));
				Scene.CreateNode(ref IID_GeoPipe, out PipeNode);
				GeoPipe = (INVGeoPipe)PipeNode;
				PipeNode.SetName(CreateNVString("Shape"));
				// Mesh mender at the end of the pipe.
				INVGeoPipeObject MeshMender = new NVGeoPipeObject_MeshMender();
				GeoPipe.AddObject(NVPIPELOCATION.PIPE_BACK, MeshMender);

				// Torus at the front.
				INVGeoPipeObject Shape = (INVGeoPipeObject)new NVGeoPipeObject_Torus();
				GeoPipe.AddObject(NVPIPELOCATION.PIPE_FRONT, Shape);
				// Apply the override material
				GeoPipe.OverrideMaterial(Media.GetHandle());
				// A transform for the shape
				NVTransformLink TransformLink;
				Guid IIDTransformLink = Marshal.GenerateGuidForType(typeof(NVTransformLinkClass));
				Scene.CreateTransform(ref IIDTransformLink, out TransformLink);
				// now create a path with four random bezier locations
				NVFloat3 tra;
				NVFloat3 trb;
				NVFloat3 trc;
				NVFloat3 trd;
				float rad = 12.0f;
				NVScatter3(rad,rnd,out tra);
				NVScatter3(rad,rnd,out trb);
				NVScatter3(rad,rnd,out trc);
				NVScatter3(rad,rnd,out trd);
				// location keyframe every five frames
				for (uint f=0; f<100; f+=5) 
				{
					NVFloat3 Trans;
					float t = (float)f / 99.0f;
					Bezier3(tra,trb,trc,trd,t,out Trans);
					TransformLink.SetTranslation(f, ref Trans);
				}
				// rotations at beginning and end
				NVQuat q0;
				NVQuat q1;
				NVRandomQuat(rnd,out q0);
				NVRandomQuat(rnd,out q1);
				TransformLink.SetRotation(0, ref q0);
				TransformLink.SetRotation(99, ref q1);
				TransformLink.SetName(CreateNVString("mytrans"));
				TransformLink.SetSeparable(1);
				TransformLink.SetAnimated(1);
				// Add to the scene
				NVTransformLink RootTransform;
				Scene.GetRootTransform(out RootTransform);
				RootTransform.AddChild(TransformLink);
				TransformLink.AddNode(PipeNode);
			}
	
			// Zoom to the extents.
			NVCamera Cam;
			Scene.GetDefaultCamera(out Cam);
			Scene.ZoomExtents(Cam, (uint)NVSCENEBOUNDS.NVSCENEBOUNDS_GEOMETRY);
			
			return 1;
		}
	}
}


